@@ -1151,6 +1151,7 @@ g_mime_header_list_prepend (GMimeHeaderList *headers, const char *name, const ch
 	g_hash_table_replace (headers->hash, header->name, header);
 	
 	if (headers->array->len > 0) {
+		args.action = GMIME_HEADER_LIST_CHANGED_ACTION_INSERTED;
 		g_ptr_array_set_size (headers->array, headers->array->len + 1);
 		
 		dest = ((unsigned char *) headers->array->pdata) + sizeof (void *);
@@ -1160,10 +1161,10 @@ g_mime_header_list_prepend (GMimeHeaderList *headers, const char *name, const ch
 		memmove (dest, src, (sizeof (void *) * n));
 		headers->array->pdata[0] = header;
 	} else {
+		args.action = GMIME_HEADER_LIST_CHANGED_ACTION_ADDED;
 		g_ptr_array_add (headers->array, header);
 	}
 	
-	args.action = GMIME_HEADER_LIST_CHANGED_ACTION_ADDED;
 	args.header = header;
 	
 	g_mime_event_emit (headers->changed, &args);
@@ -1388,6 +1389,9 @@ g_mime_header_list_remove (GMimeHeaderList *headers, const char *name)
 	g_mime_event_remove (header->changed, (GMimeEventCallback) header_changed, headers);
 	g_ptr_array_remove_index (headers->array, i);
 	g_hash_table_remove (headers->hash, name);
+
+	args.action = GMIME_HEADER_LIST_CHANGED_ACTION_REMOVED;
+	args.header = header;
 	
 	/* look for another header with the same name... */
 	while (i < headers->array->len) {
@@ -1402,9 +1406,6 @@ g_mime_header_list_remove (GMimeHeaderList *headers, const char *name)
 		i++;
 	}
 	
-	args.action = GMIME_HEADER_LIST_CHANGED_ACTION_REMOVED;
-	args.header = header;
-	
 	g_mime_event_emit (headers->changed, &args);
 	g_object_unref (header);
 	
